#!/usr/bin/env bash

#fail the whole script if a command fails
set -Eeuo pipefail 
trap cleanup SIGINT SIGTERM ERR EXIT

script_dir=$(cd "$(dirname "${BASH_SOURCE[0]}")" &>/dev/null && pwd -P)

############## Start Utility functions ##############

cleanup() {
  trap - SIGINT SIGTERM ERR EXIT
  # script cleanup here
}

logToFile() {
    MAX_LOG_LINES=200000
    LINES_TO_REMOVE=10000
    # create the log file directory if it does not exist
    if [[ ! -d "${LOGFILE%/*}" ]]
	then
		mkdir -p "${LOGFILE%/*}"
	fi
    
    # Check if the log file exists
    if [[ -f "$LOGFILE" ]]; then
        # Count the number of lines in the log file
        num_lines=$(wc -l < "$LOGFILE")

        # Check if the number of lines exceeds the limit
        if (( num_lines >= MAX_LOG_LINES )); then
            # Calculate the number of lines to remove
            lines_to_remove=$(( num_lines - MAX_LOG_LINES + LINES_TO_REMOVE ))
            # Remove the older lines from the log file
            sed -i "1,${lines_to_remove}d" "$LOGFILE"
            
            logInfo "Truncated log file $LOGFILE."
        fi
    fi

	echo -e "$1" >> "${LOGFILE}"
}

logInfo() {
    local calling_function_name=${FUNCNAME[1]}
    if [[ "${debug_flag}" -eq 1  ]]
    then
        calling_funtion_string="${calling_function_name}(): "
    else
        calling_funtion_string=""
    fi

    # check if /dev/tty is available by makeing sure STDOUT is attached to TTY
    if [[ -t 1 ]]
	then
		echo -e "[`date +"%Y-%m-%dT%H:%M:%S%z"`] ${BLUE}[Info] ${NOFORMAT}${GREEN}${calling_funtion_string}$1${NOFORMAT}" > /dev/tty
	fi
    
    logToFile "[`date +"%Y-%m-%dT%H:%M:%S%z"`] [Info] ${calling_funtion_string} $1"
}

logWarning() {
    local calling_function_name=${FUNCNAME[1]}
    if [[ "${debug_flag}" -eq 1  ]]
    then
        calling_funtion_string="${calling_function_name}(): "
    else
        calling_funtion_string=""
    fi

    # check if /dev/tty is available by makeing sure STDOUT is attached to TTY
    if [[ -t 1 ]]
	then
		echo -e "[`date +"%Y-%m-%dT%H:%M:%S%z"`] ${ORANGE}[Warning] ${NOFORMAT}${GREEN}${calling_funtion_string}$1${NOFORMAT}" > /dev/tty
	fi
    
    logToFile "[`date +"%Y-%m-%dT%H:%M:%S%z"`] [Warning] ${calling_funtion_string} $1"
}

logError() {
    local calling_function_name=${FUNCNAME[1]}
    if [[ "${debug_flag}" -eq 1  ]]
    then
        calling_funtion_string="${calling_function_name}(): "
    else
        calling_funtion_string=""
    fi

    # check if /dev/tty is available by makeing sure STDOUT is attached to TTY
    if [[ -t 1 ]]
	then
		echo -e "[`date +"%Y-%m-%dT%H:%M:%S%z"`] ${RED}[Error] ${NOFORMAT}${GREEN}${calling_funtion_string}$1${NOFORMAT}" > /dev/tty
	fi
    
    logToFile "[`date +"%Y-%m-%dT%H:%M:%S%z"`] [Error] ${calling_funtion_string} $1"
}

logDebug() {
    local calling_function_name=${FUNCNAME[1]}
    if [[ "${debug_flag}" -eq 1  ]]
    then
        calling_funtion_string="${calling_function_name}(): "
    else
        calling_funtion_string=""
    fi
    

    if [[ "${debug_flag}" -eq 1  ]]
    then
        # check if /dev/tty is available by makeing sure STDOUT is attached to TTY
        if [[ -t 1 ]]
	    then
		    echo -e "[`date +"%Y-%m-%dT%H:%M:%S%z"`] ${PURPLE}[Debug] ${NOFORMAT}${GREEN}${calling_funtion_string}$1${NOFORMAT}" > /dev/tty
	    fi
        logToFile "[`date +"%Y-%m-%dT%H:%M:%S%z"`] [Debug] ${calling_funtion_string}$1"
    fi
}

die() {
  local msg=$1
  local code=${2-1} # default exit status 1
  logError "$msg"
  exit "$code"
}

setup_colors() {
  if [[ -t 2 ]] && [[ -z "${NO_COLOR-}" ]] && [[ "${TERM-}" != "dumb" ]]; then
    NOFORMAT='\033[0m' RED='\033[0;31m' GREEN='\033[0;32m' ORANGE='\033[0;33m' BLUE='\033[0;34m' PURPLE='\033[0;35m' CYAN='\033[0;36m' YELLOW='\033[1;33m'
  else
    NOFORMAT='' RED='' GREEN='' ORANGE='' BLUE='' PURPLE='' CYAN='' YELLOW=''
  fi
}

usage() {
  cat <<EOF
IDSTower host heartbeat script, this script send heartbeats and health metrics to IDSTower periodically.
The script is normally run by Cron periodically, but can be run manually run for debugging purposes.

Usage: $(basename "${BASH_SOURCE[0]}") [-u] [-k] [-l] [-d] [-h] [--no-color]
Available options:

-u, --api-url   IDSTower API URL (default: https://IDSTower_Host/api/v1/) 
-k, --api-key   IDSTower API Authentication Key
-l, --log-file  Log file path (default: /var/log/idstower/heartbeat.log)
-d, --debug     Print script debug info
-h, --help      Print this help and exit
--no-color      Do not use colors in output

EOF
  exit
}

parse_params() {
  # default values of variables
  debug_flag=0

  while :; do
    case "${1-}" in
    -h | --help) usage ;;
    -d | --debug) debug_flag=1 ;;
    --no-color) NO_COLOR=1 ;;
    # set APIURL with the passed value
    -u | --api-url)
	  APIURL="${2-}"
	  shift
	  ;;
    # set APIKEY with the passed value
    -k | --api-key)
      APIKEY="${2-}"
      shift
      ;;
    # set LOGFILE with the passed value
    -l | --log-file)
	  LOGFILE="${2-}"
	  shift
	  ;;
    -?*) echo "ERROR: Unknown option: $1" & usage ;;
    *) break ;;
    esac
    shift
  done

  # check required params api-url and api-Key
  [[ -z "${APIURL-}" ]] && echo -e "ERROR: Missing required parameter: -u, --api-url\n" && usage
  [[ -z "${APIKEY-}" ]] && echo -e "ERROR: Missing required parameter: -k, --api-key\n" && usage

  # if log file was not set, set it to default
  [[ -z "${LOGFILE-}" ]] && LOGFILE="/var/log/idstower/heartbeat.log"

  return 0
}

getHostMetricsInJson() {

    # get host metrics
    cpu_usage=`top -b -n2 | grep "Cpu(s)" | awk '{print $2+$4 "%"}' | tail -n1`
    memory_usage=`free | awk '/Mem/{printf("%.2f%"), $3/$2*100}'`
    disk_usage=`df -h /var/log/suricata | tail -n1 | awk '{print $5}'`

    local json_request=""

    # build json string
    json_request+='"hostMetrics": {'
    json_request+='"cpu_usage": "'${cpu_usage}'",'
    json_request+='"memory_usage": "'${memory_usage}'",'
    json_request+='"disk_usage": "'${disk_usage}'"'
    json_request+='}'

    # return json string
    logDebug "host metrics: ${json_request}"
    echo "${json_request}"
}

# returns suricata binary path
getSuricataBinaryPath(){
	# check if suricata binary exists in the default path
	if [[ -f "/usr/bin/suricata" ]]
	then
		suricata_binary_path="/usr/bin/suricata"
    elif [[ -f "/sbin/suricata" ]]
    then
		suricata_binary_path="/sbin/suricata"
	elif [[ -f "/usr/sbin/suricata" ]]
    then
        suricata_binary_path="/usr/sbin/suricata"
    elif [[ -f "/usr/local/bin/suricata" ]]
	then
		suricata_binary_path="/usr/local/bin/suricata"
	elif [[ -f "/usr/local/sbin/suricata" ]]
	then
		suricata_binary_path="/usr/local/sbin/suricata"
    else
		logWarning "suricata binary not found, please install suricata or make sure it is in the default path."
        echo ""
	fi

    echo "${suricata_binary_path}"
}

# returns suricata Socket file path
getSuricataSocketFile() {

    # get the passed suricata config file path
    suricata_config_file="${1-}"

    suricata_binary_path=`getSuricataBinaryPath`
	logDebug "suricata command binary path: ${suricata_binary_path}"

    # first we check if there is a custom socket file path in the config file
    # check if the return suricata binary exists
    logDebug "checking if suricata binary exists..."
    if [[ -f "${suricata_binary_path}" ]]
    then
        logDebug "obtaining socket file path from Suricata config at: ${suricata_config_file}..."
        socket_file=`${suricata_binary_path} -c ${suricata_config_file} --dump-config | grep unix-command.filename`
        # if exit code is 0, then there is a custom socket file path in the config file
        if [[ $? -eq 0 ]]
	    then
		    # get the socket file path from the config file
		    socket_file=`echo ${socket_file} | cut -d "=" -f2 | tr -d ' '`

            # check if socket_file is empty or not
            if [[ -z "${socket_file}" ]]
			then
				logDebug "socket file not found in Suricata config file: ${suricata_config_file}"
            else
                logDebug "socket file found in Suricata config file: ${socket_file}"
			fi

            logDebug "checking if socket file exists..."
            # check if socket file exists
            if [[ -S "${socket_file}" ]]
		    then
			    logDebug "Suricata socket file found: ${socket_file}"
		    elif [[ -S "/var/run/suricata/${socket_file}" ]]
            then
                socket_file="/var/run/suricata/${socket_file}"
			    logDebug "socket file found: ${socket_file}"
            # also check for the standard socket file path before giving up
            elif [[ -S "/var/run/suricata/suricata-command.socket" ]]
		    then
			    socket_file="/var/run/suricata/suricata-command.socket"
                logDebug "default socket file found: ${socket_file}"
            else
			    logWarning "socket file not found: ${socket_file}"
                echo ""
                return 1
		    fi
        else
            logDebug "socket file not found in Suricata config file: ${suricata_config_file}"
            logDebug "checking if default socket file exists..."
            # check if default socket file exists
            if [[ -S "/var/run/suricata/suricata-command.socket" ]]
			then
				socket_file="/var/run/suricata/suricata-command.socket"
				logDebug "default socket file found: ${socket_file}"
			else
				logWarning "socket file not found: ${socket_file}"
				echo ""
				return 1
			fi
	    fi
    elif [[ -S "/var/run/suricata/suricata-command.socket" ]]
    then
        socket_file="/var/run/suricata/suricata-command.socket"
        logDebug "Suricata binary path not found, using default socket file path: ${socket_file}"
    else
        logWarning "Suricata binary path not found, please install suricata or make sure it is in the default path."
	    echo ""
	    return 1
    fi

    # after we have got the socket file path, we need to check if it is active (eg: the service is running)
    # to do this we run a simple command and observe the exit code
    # if the exit code is 0, then the socket file is active
    logDebug "checking if Suricata socket file at the following location is active: ${socket_file}"
    suricatasc -c version ${socket_file} > /dev/null 2>&1
    if [[ $? -eq 0 ]]
    then
	    logDebug "the following socket file is active: ${socket_file}"
        echo "${socket_file}"
        return 0
    else
	    logWarning "socket file is not active: ${socket_file}, make sure suricata service is running."
	    echo ""
        return 1
    fi
}

# returns suricata version
getSuricataVersion() {

    # get passed suricata config file path
    suricata_config_file="${1-}"

    logDebug "Getting Suricata version using suricatasc..."

    # get suricata socket file path
    socket_file=`getSuricataSocketFile ${suricata_config_file}`
    logDebug "Suricata socket file path: ${socket_file}"

    suricata_binary_path=`getSuricataBinaryPath`
	logDebug "suricata command binary path: ${suricata_binary_path}"

    # check if socket file is not empty & exists
    if [[ -S "${socket_file}" ]]
	then
		logDebug "Suricata socket file found: ${socket_file}"
        logDebug "Getting Suricata version using suricatasc..."
        suricata_version=`suricatasc -c version ${socket_file} | cut -d "\"" -f 4`
    elif [[ -f "${suricata_binary_path}" ]]
	then
        logWarning "No active Suricata socket file found, Suricata service might be stopped."
        logDebug "obtaining suricata version using suricata command"
        suricata_version=`${suricata_binary_path} -V | head -n1 | grep -oP '([0-9]+\.)+[0-9]+.+$'`
    else
        logWarning "Could not obtain Suricata version."
        echo ""
	fi

    # return suricata version
    logDebug "Suricata version: ${suricata_version}"
    echo "${suricata_version}"
}


# return the service status as json string
getSuricataMetricsInJson() {

	# get passed targetSoftware name
    target_software_name=$1

    # get passed targetSoftware service name
    target_software_service_name=$2

    # get passed targetSoftware config file path
	target_software_config_file_path=$3

    # get service status
    service_version=`getSuricataVersion ${target_software_config_file_path}`

    logDebug "Getting Suricata service Status..."
    service_status=`systemctl is-active ${target_software_service_name}`
    service_uptime_day=`systemctl status ${target_software_service_name} | head -n3 | tail -n1 | cut -d ' ' -f 9`
    service_uptime_hours=`systemctl status ${target_software_service_name} | head -n3 | tail -n1 | cut -d ' ' -f 10`
    service_uptime_timezone=`systemctl status ${target_software_service_name} | head -n3 | tail -n1 | cut -d ' ' -f 11 | sed 's/.$//'`

    # get suricata Socket file path
    socket_file=`getSuricataSocketFile ${target_software_config_file_path}`
    logDebug "Suricata socket file: ${socket_file}"
    
    logDebug "obtaining suricata metrics using suricatasc..."
    
    # check if socket file exists
	if [[ -S "${socket_file}" ]]
    then
        logDebug "Getting Suricata rules stats..."
        suricata_rules_stats=`suricatasc -c ruleset-stats ${socket_file}`
	    if [[ "$suricata_rules_stats" == ""  ]]; then
                suricata_rules_stats="null"
        fi

        logDebug "Getting Suricata counters dump..."
        #take two counters measurements over 15 seconds
        suricata_counters_t0=`suricatasc -c dump-counters ${socket_file}`
        if [[ "$suricata_counters_t0" == ""  ]]; then
                suricata_counters_t0="null"
        fi

        sleep 15

        suricata_counters_t1=`suricatasc -c dump-counters ${socket_file}`
        if [[ "$suricata_counters_t1" == ""  ]]; then
                suricata_counters_t1="null"
        fi

        # build suricata stats json string
        suricata_stats_json="{"
        suricata_stats_json+='"suricata_rules_stats": '${suricata_rules_stats}','
            suricata_stats_json+='"suricata_counters": {'
                suricata_stats_json+='"t0": '${suricata_counters_t0}','
                suricata_stats_json+='"t1": '${suricata_counters_t1}''
            suricata_stats_json+='}'
        suricata_stats_json+='}'

        # Log partial suricata_stats_json string
        logDebug "Suricata stats json: ${suricata_stats_json:0:100}..."

    else
        logWarning "Can not obtain suricata metrics, active socket file not found: ${socket_file}"
        # build suricata stats json string
        suricata_stats_json="null"
        logDebug "Suricata stats json: ${suricata_stats_json}"

    fi


    # define a local json string
    local json_request=""

    # build json string
    json_request+='{'
    json_request+='"name": "'${target_software_name}'",'
    json_request+='"version": "'${service_version}'",'
    json_request+='"status": "'${service_status}'",'
    json_request+='"uptime": "'${service_uptime_day}' '${service_uptime_hours}' '${service_uptime_timezone}'",'
    json_request+='"metrics": '${suricata_stats_json}''
    json_request+='}'

    logDebug "Suricata metrics json: ${json_request:0:200}..."

	# return json string
	echo "${json_request}"
}

getFilebeatMetricsInJson() {
	# get passed targetSoftware name
    target_software_name=$1

    # get passed targetSoftware service name
    target_software_service_name=$2

    # get passed targetSoftware config file path
	target_software_config_file_path=$3

    # get service status
    service_version=`filebeat version | cut -d " " -f 3`

    logDebug "Getting filebeat service Status..."
    service_status=`systemctl is-active ${target_software_service_name}`
    service_uptime_day=`systemctl status ${target_software_service_name} | head -n3 | tail -n1 | cut -d ' ' -f 9`
    service_uptime_hours=`systemctl status ${target_software_service_name} | head -n3 | tail -n1 | cut -d ' ' -f 10`
    service_uptime_timezone=`systemctl status ${target_software_service_name} | head -n3 | tail -n1 | cut -d ' ' -f 11 | sed 's/.$//'`

    logDebug "Bulding filebeat metrics json..." 
    # define a local json string
    local json_request=""

    # build json string
    json_request+='{'
    json_request+='"name": "'${target_software_name}'",'
    json_request+='"version": "'${service_version}'",'
    json_request+='"status": "'${service_status}'",'
    json_request+='"uptime": "'${service_uptime_day}' '${service_uptime_hours}' '${service_uptime_timezone}'",'
        json_request+='"metrics": null'
    json_request+='}'

    logDebug "Filebeat metrics json: ${json_request:0:200}..."

	# return json string
	echo "${json_request}"
}


############## END Utility functions ##############

############## Start Main Script ##############
parse_params "$@"
setup_colors

# API URL: {{ ApiURL }}
# API KEY: {{ ApiKey }}

# set script variables
targetSoftwaresList=([[% for software in (targetSoftwares | map(attribute='name') | list) %]] "{{ software | lower }}" [[% endfor %]])

suricata_service_name={{ Suricata_service_name | default('suricata.service') }}
suricata_config_file_path={{ Suricata_INSTALL_DIR | default('/etc/suricata/') }}suricata.yaml

filebeat_service_name={{ Filebeat_service_name | default('filebeat.service') }}
filebeat_config_file_path={{ Filebeat_config_dir | default('/etc/filebeat/') }}filebeat.yml

logInfo "==================== IDSTower Heartbeat Script ===================="
logInfo "Starting script..."

# check if curl exits
logDebug "Verifying curl command exists..."
if ! command -v curl &> /dev/null
then
	logError "curl command could not be found, please install it and try again."
	exit 1
fi

# count number of targetSoftwaresNames
num_targetSoftwares=0
for software in "${targetSoftwaresList[@]}"
do
		let num_targetSoftwares=num_targetSoftwares+1
done

# start building the json request
json_request="{"
json_request+="\"apikey\": \"${APIKEY}\","

# add host metrics to json request
logDebug "Getting host metrics..."
host_metrics_json=`getHostMetricsInJson`
json_request+="${host_metrics_json}"

# check if num_targetSoftwares is greater than 0
if [[ ${num_targetSoftwares} -gt 0 ]]
then
    json_request+=","    
fi

#services status
json_request+=' "servicesStatus": ['
i=0
for targetSoftware in "${targetSoftwaresList[@]}"
do
        let i=i+1
        logInfo "Getting ${targetSoftware} metrics..."
        #set target software service name
        if [[ "$targetSoftware" == "suricata" ]]
		then
				targetSoftwareServiceName=${suricata_service_name}
                suricata_metrics_json=`getSuricataMetricsInJson ${targetSoftware} ${targetSoftwareServiceName} ${suricata_config_file_path}`
                json_request+="${suricata_metrics_json}"
		elif [[ "$targetSoftware" == "filebeat" ]]
		then
				targetSoftwareServiceName=${filebeat_service_name}
                filebeat_metrics_json=`getFilebeatMetricsInJson ${targetSoftware} ${targetSoftwareServiceName} ${filebeat_config_file_path}`
				json_request+="${filebeat_metrics_json}"
		else
				die "Unknown target software: ${targetSoftware}" 
		fi

        if [ $i -ne $num_targetSoftwares ]; then
                json_request+=","
        fi
done
json_request+=']'

json_request+="}"

# save the json request to a file
logDebug "Saving json request to a temporary file: /tmp/idstower_heartbeat.tmp"
echo "${json_request}" > /tmp/idstower_heartbeat.tmp

# send the request
logInfo "Sending heartbeat to IDSTower..."
curlDebugParam=""
if [[ "${debug_flag}" -eq 1 ]]
then
	curlDebugParam=" -v"
fi

apiResponse=`curl ${curlDebugParam} -L -s -k --max-time 10 -X POST -H "Content-Type: application/json" --data "@/tmp/idstower_heartbeat.tmp" ${APIURL}hosts/heartbeat || true`

# check if api response is not empty
if [[ -z "${apiResponse}" ]]
then
	logError "Failed to send heartbeat to IDSTower, Make sure IDSTower service is running, to debug this issue, run the script with -d flag."
else
    logInfo "Heartbeat sent successfully to IDSTower, API Response: ${apiResponse}"
fi

#delete the json request file
logDebug "Deleting temporary file: /tmp/idstower_heartbeat.tmp"
rm -rf /tmp/idstower_heartbeat.tmp

logInfo "IDSTower Heartbeat finished."