#!/usr/bin/env bash

#fail the whole script if a command fails
set -Eeuo pipefail 
trap cleanup SIGINT SIGTERM ERR EXIT

script_dir=$(cd "$(dirname "${BASH_SOURCE[0]}")" &>/dev/null && pwd -P)

############## Start Utility functions ##############
cleanup() {
  trap - SIGINT SIGTERM ERR EXIT
  # script cleanup here
}

logToFile() {
    MAX_LOG_LINES=200000
    LINES_TO_REMOVE=10000
    # create the log file directory if it does not exist
    if [[ ! -d "${LOGFILE%/*}" ]]
	then
		mkdir -p "${LOGFILE%/*}"
	fi
    
    # Check if the log file exists
    if [[ -f "$LOGFILE" ]]; then
        # Count the number of lines in the log file
        num_lines=$(wc -l < "$LOGFILE")

        # Check if the number of lines exceeds the limit
        if (( num_lines >= MAX_LOG_LINES )); then
            # Calculate the number of lines to remove
            lines_to_remove=$(( num_lines - MAX_LOG_LINES + LINES_TO_REMOVE ))
            # Remove the older lines from the log file
            sed -i "1,${lines_to_remove}d" "$LOGFILE"
            
            logInfo "Truncated log file $LOGFILE."
        fi
    fi

	echo -e "$1" >> "${LOGFILE}"
}
logInfo() {
    local calling_function_name=${FUNCNAME[1]}
    if [[ "${debug_flag}" -eq 1  ]]
    then
        calling_funtion_string="${calling_function_name}(): "
    else
        calling_funtion_string=""
    fi

    # check if /dev/tty is available by makeing sure STDOUT is attached to TTY
    if [[ -t 1 ]]
	then
		echo -e "[`date +"%Y-%m-%dT%H:%M:%S%z"`] ${BLUE}[Info] ${NOFORMAT}${GREEN}${calling_funtion_string}$1${NOFORMAT}" > /dev/tty
	fi
    
    logToFile "[`date +"%Y-%m-%dT%H:%M:%S%z"`] [Info] ${calling_funtion_string} $1"
}

logWarning() {
    local calling_function_name=${FUNCNAME[1]}
    if [[ "${debug_flag}" -eq 1  ]]
    then
        calling_funtion_string="${calling_function_name}(): "
    else
        calling_funtion_string=""
    fi

    # check if /dev/tty is available by makeing sure STDOUT is attached to TTY
    if [[ -t 1 ]]
	then
		echo -e "[`date +"%Y-%m-%dT%H:%M:%S%z"`] ${ORANGE}[Warning] ${NOFORMAT}${GREEN}${calling_funtion_string}$1${NOFORMAT}" > /dev/tty
	fi
    
    logToFile "[`date +"%Y-%m-%dT%H:%M:%S%z"`] [Warning] ${calling_funtion_string} $1"
}

logError() {
    local calling_function_name=${FUNCNAME[1]}
    if [[ "${debug_flag}" -eq 1  ]]
    then
        calling_funtion_string="${calling_function_name}(): "
    else
        calling_funtion_string=""
    fi

    # check if /dev/tty is available by makeing sure STDOUT is attached to TTY
    if [[ -t 1 ]]
	then
		echo -e "[`date +"%Y-%m-%dT%H:%M:%S%z"`] ${RED}[Error] ${NOFORMAT}${GREEN}${calling_funtion_string}$1${NOFORMAT}" > /dev/tty
	fi
    
    logToFile "[`date +"%Y-%m-%dT%H:%M:%S%z"`] [Error] ${calling_funtion_string} $1"
}

logDebug() {
    local calling_function_name=${FUNCNAME[1]}
    if [[ "${debug_flag}" -eq 1  ]]
    then
        calling_funtion_string="${calling_function_name}(): "
    else
        calling_funtion_string=""
    fi
    

    if [[ "${debug_flag}" -eq 1  ]]
    then
        # check if /dev/tty is available by makeing sure STDOUT is attached to TTY
        if [[ -t 1 ]]
	    then
		    echo -e "[`date +"%Y-%m-%dT%H:%M:%S%z"`] ${PURPLE}[Debug] ${NOFORMAT}${GREEN}${calling_funtion_string}$1${NOFORMAT}" > /dev/tty
	    fi
        logToFile "[`date +"%Y-%m-%dT%H:%M:%S%z"`] [Debug] ${calling_funtion_string}$1"
    fi
}

die() {
  local msg=$1
  local code=${2-1} # default exit status 1
  logError "$msg"
  exit "$code"
}

setup_colors() {
  if [[ -t 2 ]] && [[ -z "${NO_COLOR-}" ]] && [[ "${TERM-}" != "dumb" ]]; then
    NOFORMAT='\033[0m' RED='\033[0;31m' GREEN='\033[0;32m' ORANGE='\033[0;33m' BLUE='\033[0;34m' PURPLE='\033[0;35m' CYAN='\033[0;36m' YELLOW='\033[1;33m'
  else
    NOFORMAT='' RED='' GREEN='' ORANGE='' BLUE='' PURPLE='' CYAN='' YELLOW=''
  fi
}

usage() {
  cat <<EOF
IDSTower Suricata Indicators of Compromise (IOCs) update script, this script download latest IOCs from IDSTower periodically and applies them to Suricata.
The script is normally run by Cron periodically, but can be run manually run for debugging purposes.

Usage: $(basename "${BASH_SOURCE[0]}") [-u] [-k] [-f] [-s] [-l] [-d] [-h] [--no-color]
Available options:

-u, --api-url   IDSTower API URL (default: https://IDSTower_Host/api/v1/) 
-k, --api-key   IDSTower API Authentication Key
-f, --force     Force Update even if we have the latest Rules version locally
-s, --skip-delay Skips the initial random delay before running the update
-l, --log-file  Log file path (default: /var/log/idstower/indicators_updates.log)
-d, --debug     Print script debug info
-h, --help      Print this help and exit
--no-color      Do not use colors in output

EOF
  exit
}

parse_params() {
  # default values of variables
  debug_flag=0
  force_flag=0
  skip_delay_flag=0

  while :; do
    case "${1-}" in
    -h | --help) usage ;;
    -d | --debug) debug_flag=1 ;;
    -f | --force) force_flag=1 ;;
    -s | --skip-delay) skip_delay_flag=1 ;;
    --no-color) NO_COLOR=1 ;;
    # set APIURL with the passed value
    -u | --api-url)
	  APIURL="${2-}"
	  shift
	  ;;
    # set APIKEY with the passed value
    -k | --api-key)
      APIKEY="${2-}"
      shift
      ;;
    # set LOGFILE with the passed value
    -l | --log-file)
	  LOGFILE="${2-}"
	  shift
	  ;;
    -?*) echo "ERROR: Unknown option: $1" & usage ;;
    *) break ;;
    esac
    shift
  done

  # check required params api-url and api-Key
  [[ -z "${APIURL-}" ]] && echo -e "ERROR: Missing required parameter: -u, --api-url\n" && usage
  [[ -z "${APIKEY-}" ]] && echo -e "ERROR: Missing required parameter: -k, --api-key\n" && usage

  # if log file was not set, set it to default
  [[ -z "${LOGFILE-}" ]] && LOGFILE="/var/log/idstower/indicators_updates.log"

  return 0
}

apiPost(){
    logDebug "sending HTTP Post request to API"
    
    #json request
    json_request="{\"apikey\": \"${APIKEY}\"}"
    logDebug "API Endpoint: ${APIURL}${1}"
    logDebug "Json Request: ${json_request}"

    #set curl debug option is debug mode is enabled
    curlDebugParmeter=""
    if [[ "${debug_flag}" -eq 1  ]]
    then
            curlDebugParmeter="-v"
    fi

    # send the request
    local apiOutput=""
    apiOutput=$(curl ${curlDebugParmeter} -L -s -k --connect-timeout 10 -X POST -H "Content-Type: application/json" --data "${json_request}" ${APIURL}${1} || true) #use `|| true` to ignore curl failures 

    logDebug "API Output: ${apiOutput}"
    

    #make sure that we are getting non-empty response from the api
    if [[ -z "$apiOutput" ]]
    then
        die "got empty response from IDSTower API, make sure IDSTower service is running correctly, to debug this issue, re-run this script with --debug option, exiting..."
    fi

    echo "${apiOutput}"
}

# function to check if indicators alerting feature is enabled or not
isIndicatorsAlertingFeatureEnabled(){
	# get passed api Output
    local apiOutput=$1

	#check api output for feature disabled message
    if [[ "${apiOutput}" == *"#Disabled: Indicators Alerting is not Enabled"* ]]
    then
        echo "no"
    fi

    echo "yes"
}

#IPs & CIDRs
updateIPrepCategories(){
    apiOutput=`apiPost "hosts/suricata/getIPrepCategories"`

    local categoriesFile=${suricata_config_dir}iprep/categories.txt
    logDebug "Local IPRep Categories file: ${categoriesFile}"

    localIPrepCategoriesFileHeader=$(head -n 1 ${categoriesFile})

    feature_enabled=`isIndicatorsAlertingFeatureEnabled "${apiOutput}"`
    logDebug "Indicators Alerting Feature Enabled: ${feature_enabled}"

    #verify that Indicators alerting is enabled in IDSTower
    if [[ "${feature_enabled}" == *"no"* ]]
    then
            logInfo "Indicators Alerting is Disabled in IDSTower, skipping update..."

            if [[ $localIPrepCategoriesFileHeader != *"#Feature is Disabled"* ]]
            then
                logInfo "Clearing Categories file..."
                printf "%s" "#Feature is Disabled in IDSTower UI, it can be enabled via IDSTower UI --> Settings --> Indicators" > "${categoriesFile}"
                restartSuricata=1
            fi

            return
    fi

    #verify the apiOutput by making sure it contains the header "#IDSTower Auto-generated IP Reputation Categories"
    if [[ $apiOutput != *"#IDSTower Auto-generated IP Reputation Categories"* ]]
    then
        die "got unexpected response from IDSTower API, make sure IDSTower service is running correctly, to debug this issue, re-run this script with --debug option, exiting..."
    fi

    #write the api output to a temporary file
    printf "%s" "$apiOutput" > "${categoriesFile}.temporary"

    #calculate the api output file hash values
    apiOutputHash=$(md5sum ${categoriesFile}.temporary | cut -d " " -f1)
    logDebug "Api Output hash: ${apiOutputHash}"
    
    #remove the temporary file
    rm -rf ${categoriesFile}.temporary

    #make sure the Categories file exist
    touch ${categoriesFile}

    #calculate the local IP Rep Categories file hash
    localIPrepCategoriesFileHash=$(md5sum ${categoriesFile} | cut -d " " -f1)
    logDebug "Local IP Reputation Categories file hash: ${localIPrepCategoriesFileHash}"

    if [[ $apiOutputHash == $localIPrepCategoriesFileHash ]]
    then
        #exit only if force_flag is not set
        if [[ $force_flag -ne 1 ]]
        then
            logInfo "Local IP Reputation Categories file is up-to-date: ${categoriesFile}"
            return 0
        fi
    fi

    printf "%s" "$apiOutput" > "${categoriesFile}"

    if [[ $? -ne 0 ]]
    then
        die "Failed to update Local IP Reputation Categories file, could not write to: ${categoriesFile}, exiting script."
    fi

    #since the category file was updated, we need to restart Suricata
    #we set the flag so suricata get restarted at the end of the script
    restartSuricata=1
    logInfo "Local IP Reputation Categories file updated successfully: ${categoriesFile}"
}

updateIPrepIndicatorsList(){
    apiOutput=`apiPost "hosts/suricata/getIPrepIndicatros"`

    local indicatorsFile=${suricata_config_dir}iprep/idstower_ip_indicators.list
    logDebug "Local IP indicators file: ${indicatorsFile}"

    localIpIndicatorsFileHeader=$(head -n 1 ${indicatorsFile})
    
    feature_enabled=`isIndicatorsAlertingFeatureEnabled "${apiOutput}"`
    logDebug "Indicators Alerting Feature Enabled: ${feature_enabled}"

    #verify that Indicators alerting is enabled in IDSTower
    if [[ "${feature_enabled}" == *"no"* ]]
    then
            logInfo "Indicators Alerting is Disabled in IDSTower, skipping update..."
            if [[ $localIpIndicatorsFileHeader != *"#Feature is Disabled"* ]]
            then
                logInfo "Clearing IP indicators file..."
                printf "%s" "#Feature is Disabled in IDSTower UI, it can be enabled via IDSTower UI --> Settings --> Indicators" > "${indicatorsFile}"
                triggerSuricataRulesReload=1
            fi

            return
    fi

    #verify the apiOutput by making sure it contains the header "#IDSTower Auto-generated IP Reputation Indicators"
    if [[ $apiOutput != *"#IDSTower Auto-generated IP Reputation Indicators"* ]]
    then
        die "got unexpected response from IDSTower API, make sure IDSTower service is running correctly, to debug this issue, re-run this script with --debug option, exiting..."
    fi
    
    #write the api output to a temporary file
    printf "%s" "$apiOutput" > "${indicatorsFile}.temporary"

    #calculate the api output file hash values
    apiOutputHash=$(md5sum ${indicatorsFile}.temporary | cut -d " " -f1)
    logDebug "Api Output hash: ${apiOutputHash}"
    
    #remove the temporary file
    rm -rf ${indicatorsFile}.temporary

    #make sure the indicators file exist
    touch ${indicatorsFile}

    #calculate the local IP Rep Indicators file hash
    localIPrepIndicatorsFileHash=$(md5sum ${indicatorsFile} | cut -d " " -f1)
    logDebug "Local IP Reputation Indicators file hash: ${localIPrepIndicatorsFileHash}"

    if [[ $apiOutputHash == $localIPrepIndicatorsFileHash ]]
    then
        #exit only if force_flag is not set
        if [[ $force_flag -ne 1 ]]
        then
            logInfo "Local IP Reputation Indicators file is up-to-date: ${indicatorsFile}"
            return 0
        fi
    fi

    printf "%s" "$apiOutput" > "${indicatorsFile}"
    
    if [[ $? -ne 0 ]]
    then
        die "Failed to update Local IP Reputation Indicators file, could not write to: ${indicatorsFile}, exiting script."
    fi

    #since the indicators file was updated, we need to reload Suricata
    #we set the flag so suricata gets reloaded  at the end of the script
    triggerSuricataRulesReload=1
    logInfo "Local IP Reputation Indicators file updated successfully: ${indicatorsFile}"
}

updateIPrepRulesList(){
    apiOutput=`apiPost "hosts/suricata/getIPrepRules"`

    local rulesFile=${suricata_config_dir}rules/idstower_iprep_indicators.rules
    logDebug "Local IPRep Rules file: ${rulesFile}"

    localIPrepRulesFileHeader=$(head -n 1 ${rulesFile})

    feature_enabled=`isIndicatorsAlertingFeatureEnabled "${apiOutput}"`
    logDebug "Indicators Alerting Feature Enabled: ${feature_enabled}"

    #verify that Indicators alerting is enabled in IDSTower
    if [[ "${feature_enabled}" == *"no"* ]]
    then
            logInfo "Indicators Alerting is Disabled in IDSTower, skipping update..."
            if [[ $localIPrepRulesFileHeader != *"#Feature is Disabled"* ]]
            then
                logInfo "Clearing IPRep Rules file..."
                printf "%s" "#Feature is Disabled in IDSTower UI, it can be enabled via IDSTower UI --> Settings --> Indicators" > "${rulesFile}"
                triggerSuricataRulesReload=1
            fi

            return
    fi

    #verify the apiOutput by making sure it contains the header "#IDSTower Auto-generated IP Reputation Rules"
    if [[ $apiOutput != *"#IDSTower Auto-generated IP Reputation Rules"* ]]
    then
        die "got unexpected response from IDSTower API, make sure IDSTower service is running correctly, to debug this issue, re-run this script with --debug option, exiting..."
    fi

    #write the api output to a temporary file
    printf "%s" "$apiOutput" > "${rulesFile}.temporary"

    #calculate the api output file hash values
    apiOutputHash=$(md5sum ${rulesFile}.temporary | cut -d " " -f1)
    logDebug "Api Output hash: ${apiOutputHash}"
    
    #remove the temporary file
    rm -rf ${rulesFile}.temporary

    #make sure the rules file exist
    touch ${rulesFile}

    #calculate the local IP Rep rules file hash
    localIPrepRulesFileHash=$(md5sum ${rulesFile} | cut -d " " -f1)
    logDebug "Local IP Reputation rules file hash: ${localIPrepRulesFileHash}"

    if [[ $apiOutputHash == $localIPrepRulesFileHash ]]
    then
        #exit only if force_flag is not set
        if [[ $force_flag -ne 1 ]]
        then
            logInfo "Local IP Reputation Rules file is up-to-date: ${rulesFile}"
            return 0
        fi
    fi

    printf "%s" "$apiOutput" > "${rulesFile}"
    
    if [[ $? -ne 0 ]]
    then
        die "Failed to update Local IP Reputation Rules file, could not write to: ${rulesFile}, exiting script."
    fi

    #since the IP Reputation Rules file was updated, we need to reload Suricata
    #we set the flag so suricata gets reloaded  at the end of the script
    triggerSuricataRulesReload=1
    logInfo "Local IP Reputation Rules file updated successfully: ${rulesFile}"
}

#FQDNs
updateDatasetsFQDNsIndicatorsList(){
    apiOutput=`apiPost "hosts/suricata/getDatasetsFQDNsIndicators"`

    indicatorsFile=${suricata_config_dir}datasets/idstower_fqdn_indicators.list
    logDebug "Local FQDN indicators file: ${indicatorsFile}"

    localIndicatorsFileLinesCount=$(wc -l ${indicatorsFile} | cut -d " " -f1)

    feature_enabled=`isIndicatorsAlertingFeatureEnabled "${apiOutput}"`
    logDebug "Indicators Alerting Feature Enabled: ${feature_enabled}"

    #verify that Indicators alerting is enabled in IDSTower
    if [[ "${feature_enabled}" == *"no"* ]]
    then
            logInfo "Indicators Alerting is Disabled in IDSTower, skipping update..."
            #make sure the file is empty as it dose not accept a header
            if [[ $localIndicatorsFileLinesCount != "0" ]]
            then
                logInfo "Clearing FQDN indicators file..."
                printf "%s" "" > "${indicatorsFile}"
                triggerSuricataRulesReload=1
            fi

            return
    fi

    #verify the apiOutput by making sure it contains the header "#IDSTower Auto-generated Datasets Indicators for type: FQDN"
    if [[ $apiOutput != *"#IDSTower Auto-generated Datasets Indicators for type: FQDN"* ]]
    then
        die "got unexpected response from IDSTower API, make sure IDSTower service is running correctly, to debug this issue, re-run this script with --debug option, exiting..."
    fi

    #write the api output to a temporary file
    #we manually write a new line "\n" at the end of the file, otherwise suricata will fail to parse the file
    #we also remove the header (using tail -n +2) since suricata will fail to parse it
    printf "%s\n" "$apiOutput" | tail -n +2 > "${indicatorsFile}.temporary"

    #calculate the api output file hash values
    apiOutputHash=$(md5sum ${indicatorsFile}.temporary | cut -d " " -f1)
    logDebug "Api Output hash: ${apiOutputHash}"
    
    #remove the temporary file
    rm -rf ${indicatorsFile}.temporary

    #make sure the indicators file exist
    touch ${indicatorsFile}

    #calculate the local FQDNs Indicators file hash
    localFQDNsIndicatorsFileHash=$(md5sum ${indicatorsFile} | cut -d " " -f1)
    logDebug "Local FQDNs Indicators file hash: ${localIPrepIndicatorsFileHash}"

    if [[ $apiOutputHash == $localFQDNsIndicatorsFileHash ]]
    then
        #exit only if force_flag is not set
        if [[ $force_flag -ne 1 ]]
        then
            logInfo "Local FQDNs Indicators file is up-to-date: ${indicatorsFile}"
            return 0
        fi
    fi

    #we manually wirte a new line "\n" at the end of the file, otherwise suricata will fail to parse the file
    printf "%s\n" "$apiOutput" | tail -n +2 > "${indicatorsFile}"

    if [[ $? -ne 0 ]]
    then
        die "Failed to update Local FQDNs Indicators file, could not write to: ${indicatorsFile}, exiting script."
    fi

    #since the indicators file was updated, we need to reload Suricata
    #we set the flag so suricata gets reloaded  at the end of the script
    triggerSuricataRulesReload=1
    logInfo "Local FQDNs Indicators file updated successfully: ${indicatorsFile}"
}

#SHA256
updateDatasetsSHA256IndicatorsList(){
    apiOutput=`apiPost "hosts/suricata/getDatasetsSHA256Indicators/${1}"`

    local indicatorsFile=${suricata_config_dir}datasets/idstower_sha256_indicators_score${1}.list
    logDebug "Local SHA256 indicators file: ${indicatorsFile}"

    localIndicatorsFileHeader=$(head -n 1 ${indicatorsFile})

    feature_enabled=`isIndicatorsAlertingFeatureEnabled "${apiOutput}"`
    logDebug "Indicators Alerting Feature Enabled: ${feature_enabled}"

    #verify that Indicators alerting is enabled in IDSTower
    if [[ "${feature_enabled}" == *"no"* ]]
    then
            logInfo "Indicators Alerting is Disabled in IDSTower, skipping update..."
            
            if [[ $localIndicatorsFileHeader != *"#Feature is Disabled"* ]]
            then
                logInfo "Clearing SHA256 indicators file..."
                printf "%s" "#Feature is Disabled in IDSTower UI, it can be enabled via IDSTower UI --> Settings --> Indicators" > "${indicatorsFile}"
                triggerSuricataRulesReload=1
            fi

            return
    fi

    #verify the apiOutput by making sure it contains the header "#IDSTower Auto-generated Datasets Indicators for type: SHA256"
    if [[ $apiOutput != *"#IDSTower Auto-generated Datasets Indicators for type: SHA256"* ]]
    then
        die "got unexpected response from IDSTower API, make sure IDSTower service is running correctly, to debug this issue, re-run this script with --debug option, exiting..."
    fi

    #write the api output to a temporary file
    #we manually write a new line "\n" at the end of the file
    printf "%s\n" "$apiOutput" > "${indicatorsFile}.temporary"

    #calculate the api output file hash values
    apiOutputHash=$(md5sum ${indicatorsFile}.temporary | cut -d " " -f1)
    logDebug "Api Output hash: ${apiOutputHash}"
    
    #remove the temporary file
    rm -rf ${indicatorsFile}.temporary

    #make sure the indicators file exist
    touch ${indicatorsFile}

    #calculate the local SHA256 Indicators file hash
    localSHA256IndicatorsFileHash=$(md5sum ${indicatorsFile} | cut -d " " -f1)
    logDebug "Local SHA256 Indicators of score ${1} file hash: ${localIPrepIndicatorsFileHash}"

    if [[ $apiOutputHash == $localSHA256IndicatorsFileHash ]]
    then
        #exit only if force_flag is not set
        if [[ $force_flag -ne 1 ]]
        then
            logInfo "Local SHA256 Indicators of score ${1} file is up-to-date: ${indicatorsFile}"
            return 0
        fi
    fi

    #we manually wirte a new line "\n" at the end of the file
    printf "%s\n" "$apiOutput" > "${indicatorsFile}"
    
    if [[ $? -ne 0 ]]
    then
        die "Failed to update Local SHA256 Indicators of score ${1} file, could not write to: ${indicatorsFile}, exiting script."
    fi

    #since the indicators file was updated, we need to reload Suricata
    #we set the flag so suricata gets reloaded  at the end of the script
    triggerSuricataRulesReload=1
    logInfo "Local SHA256 Indicators of score ${1} file updated successfully: ${indicatorsFile}"
}

updateDatasetsRulesList(){
    apiOutput=`apiPost "hosts/suricata/getDatasetsRules"`

    local rulesFile=${suricata_config_dir}rules/idstower_datasets_indicators.rules
    logDebug "Local Indicators Rules file: ${rulesFile}"

    localRulesFileHeader=$(head -n 1 ${rulesFile})

    feature_enabled=`isIndicatorsAlertingFeatureEnabled "${apiOutput}"`
    logDebug "Indicators Alerting Feature Enabled: ${feature_enabled}"

    #verify that Indicators alerting is enabled in IDSTower
    if [[ "${feature_enabled}" == *"no"* ]]
    then
            logInfo "Indicators Alerting is Disabled in IDSTower, skipping update..."

            if [[ $localRulesFileHeader != *"#Feature is Disabled"* ]]
            then
                logInfo "Clearing SHA256 indicators file..."
                printf "%s" "#Feature is Disabled in IDSTower UI, it can be enabled via IDSTower UI --> Settings --> Indicators" > "${rulesFile}"
                triggerSuricataRulesReload=1
            fi

            return
    fi

    #verify the apiOutput by making sure it contains the header "#IDSTower Auto-generated IP Reputation Rules"
    if [[ $apiOutput != *"#IDSTower Auto-generated Datasets Rules"* ]]
    then
        die "got unexpected response from IDSTower API, make sure IDSTower service is running correctly, to debug this issue, re-run this script with --debug option, exiting..."
    fi

    #write the api output to a temporary file
    printf "%s" "$apiOutput" > "${rulesFile}.temporary"

    #calculate the api output file hash values
    apiOutputHash=$(md5sum ${rulesFile}.temporary | cut -d " " -f1)
    logDebug "Api Output hash: ${apiOutputHash}"
    
    #remove the temporary file
    rm -rf ${rulesFile}.temporary

    #make sure the rules file exist
    touch ${rulesFile}

    #calculate the local Datasets rules file hash
    localDatasetsRulesFileHash=$(md5sum ${rulesFile} | cut -d " " -f1)
    logDebug "Local Datasets rules file hash: ${localDatasetsRulesFileHash}"

    if [[ $apiOutputHash == $localDatasetsRulesFileHash ]]
    then
        #exit only if force_flag is not set
        if [[ $force_flag -ne 1 ]]
        then
            logInfo "Local Datasets Rules file is up-to-date: ${rulesFile}"
            return 0
        fi
    fi

    printf "%s" "$apiOutput" > "${rulesFile}"

    if [[ $? -ne 0 ]]
    then
        die "Failed to update Local Datasets Rules file, could not write to: ${rulesFile}, exiting script."
    fi

    #since the Datasets Rules file was updated, we need to reload Suricata
    #we set the flag so suricata gets reloaded  at the end of the script
    triggerSuricataRulesReload=1
    logInfo "Local Datasets Rules file updated successfully: ${rulesFile}"
}

# returns suricata binary path
getSuricataBinaryPath(){
	# check if suricata binary exists in the default path
	if [[ -f "/usr/bin/suricata" ]]
	then
		suricata_binary_path="/usr/bin/suricata"
    elif [[ -f "/sbin/suricata" ]]
    then
		suricata_binary_path="/sbin/suricata"
	elif [[ -f "/usr/sbin/suricata" ]]
    then
        suricata_binary_path="/usr/sbin/suricata"
    elif [[ -f "/usr/local/bin/suricata" ]]
	then
		suricata_binary_path="/usr/local/bin/suricata"
	elif [[ -f "/usr/local/sbin/suricata" ]]
	then
		suricata_binary_path="/usr/local/sbin/suricata"
    else
		logWarning "suricata binary not found, please install suricata or make sure it is in the default path."
        echo ""
	fi

    echo "${suricata_binary_path}"
}

# returns suricata Socket file path
getSuricataSocketFile() {

    # get the passed suricata config file path
    suricata_config_file="${1-}"

    suricata_binary_path=`getSuricataBinaryPath`
	logDebug "suricata command binary path: ${suricata_binary_path}"

    # first we check if there is a custom socket file path in the config file
    # check if the return suricata binary exists
    logDebug "checking if suricata binary exists..."
    if [[ -f "${suricata_binary_path}" ]]
    then
        logDebug "obtaining socket file path from Suricata config at: ${suricata_config_file}..."
        socket_file=`${suricata_binary_path} -c ${suricata_config_file} --dump-config | grep unix-command.filename`
        # if exit code is 0, then there is a custom socket file path in the config file
        if [[ $? -eq 0 ]]
	    then
		    # get the socket file path from the config file
		    socket_file=`echo ${socket_file} | cut -d "=" -f2 | tr -d ' '`

            # check if socket_file is empty or not
            if [[ -z "${socket_file}" ]]
			then
				logDebug "socket file not found in Suricata config file: ${suricata_config_file}"
            else
                logDebug "socket file found in Suricata config file: ${socket_file}"
			fi

            logDebug "checking if socket file exists..."
            # check if socket file exists
            if [[ -S "${socket_file}" ]]
		    then
			    logDebug "Suricata socket file found: ${socket_file}"
		    elif [[ -S "/var/run/suricata/${socket_file}" ]]
            then
                socket_file="/var/run/suricata/${socket_file}"
			    logDebug "socket file found: ${socket_file}"
            # also check for the standard socket file path before giving up
            elif [[ -S "/var/run/suricata/suricata-command.socket" ]]
		    then
			    socket_file="/var/run/suricata/suricata-command.socket"
                logDebug "default socket file found: ${socket_file}"
            else
			    logWarning "socket file not found: ${socket_file}"
                echo ""
                return 1
		    fi
        else
            logDebug "socket file not found in Suricata config file: ${suricata_config_file}"
            logDebug "checking if default socket file exists..."
            # check if default socket file exists
            if [[ -S "/var/run/suricata/suricata-command.socket" ]]
			then
				socket_file="/var/run/suricata/suricata-command.socket"
				logDebug "default socket file found: ${socket_file}"
			else
				logWarning "socket file not found: ${socket_file}"
				echo ""
				return 1
			fi
	    fi
    elif [[ -S "/var/run/suricata/suricata-command.socket" ]]
    then
        socket_file="/var/run/suricata/suricata-command.socket"
        logDebug "Suricata binary path not found, using default socket file path: ${socket_file}"
    else
        logWarning "Suricata binary path not found, please install suricata or make sure it is in the default path."
	    echo ""
	    return 1
    fi

    # after we have got the socket file path, we need to check if it is active (eg: the service is running)
    # to do this we run a simple command and observe the exit code
    # if the exit code is 0, then the socket file is active
    logDebug "checking if Suricata socket file at the following location is active: ${socket_file}"
    suricatasc -c version ${socket_file} > /dev/null 2>&1
    if [[ $? -eq 0 ]]
    then
	    logDebug "the following socket file is active: ${socket_file}"
        echo "${socket_file}"
        return 0
    else
	    logWarning "socket file is not active: ${socket_file}, make sure suricata service is running."
	    echo ""
        return 1
    fi
}

restartSuricataService(){
    #we will only attempt to restart suricata if the service is already running
    #we do this to prevent the script from starting suricata service while it was in stopped state
    serviceStatus=$(systemctl show -p ActiveState ${suricata_service_name})

    if [[ "$serviceStatus" != "ActiveState=active" ]]
    then
        logInfo "Suricata service is not running, skipping restart."
        return 0
    fi

    systemctl restart ${suricata_service_name}
    
    if [[ $? -ne 0 ]]
    then
        logError "Failed to restart Suricata service, to investigate why restart failed, run: sudo systemctl status ${suricata_service_name} -l , Or run: sudo journalctl -ex -u ${suricata_service_name}"
        return 1
    fi

    logInfo "Suricata service restarted successfully."
    return 0
}

reloadSuricataRules(){

    # get passed suricata config file path
    suricata_config_file_path=$1

    #we will only attempt to reload suricata if the service is already running
    #we do this to prevent the script from starting suricata service while it was in stopped state
    serviceStatus=$(systemctl show -p ActiveState ${suricata_service_name})

    if [[ "$serviceStatus" != "ActiveState=active" ]]
    then
        logInfo "Suricata service is not running, skipping reload."
        return 0
    fi

    socket_file=`getSuricataSocketFile ${suricata_config_file_path}`

    suricatasc -c "ruleset-reload-rules" ${socket_file} > /dev/null 2>&1
    
    if [[ $? -ne 0 ]]
    then
        logError "Failed to reload Suricata rules, to investigate why reload failed, run: sudo systemctl status ${suricata_service_name} -l , Or run: sudo journalctl -ex -u ${suricata_service_name} and view Suricata logs."
        return 1
    fi

    logInfo "Suricata service reloaded successfully."
    return 0
}

############## END Utility functions ##############

############## Start Main Script ##############
parse_params "$@"
setup_colors

logInfo "==================== Suricata indicators update script ===================="
logInfo "Starting script..."

# API URL: {{ ApiURL }}
# API KEY: {{ ApiKey }}
suricata_service_name={{ Suricata_service_name | default('suricata.service') }}
suricata_config_dir={{ Suricata_INSTALL_DIR | default('/etc/suricata/') }}
suricata_config_file_path={{ Suricata_INSTALL_DIR | default('/etc/suricata/') }}suricata.yaml

#flag to tell if we need to restart suricata
restartSuricata=0
#flag to tell if we need to reload suricata Indicators
triggerSuricataRulesReload=0

#skip the random delay if skip_delay_flag is set
if [[ $skip_delay_flag -eq 0 ]]
then
    logInfo "Sleeping for up to 120 seconds to prevent overwhelming IDSTower, to skip this, re-run script with --skip-delay"
    sleep $(shuf -i 0-120 -n 1)
fi

### Update IP/CIDR based Indicators to IPRep files ###

#update the IP Reputation Categories is needed
logInfo "Updating Local IP Reputation Categories file..."
updateIPrepCategories

#update the IP Reputation indicators list
logInfo "Updating Local IP Reputation Indicators list file..."
updateIPrepIndicatorsList

#update the IP Reputation indicators alert rules
logInfo "Updating Local IP Reputation Rules list file..."
updateIPrepRulesList

#update the Datasets FQDNs indicators list
logInfo "Updating FQDNs Indicators list file..."
updateDatasetsFQDNsIndicatorsList

#update the Datasets SHA256 indicators list
logInfo "Updating SHA256 Indicators list files..."
for score in {1..10}
do
    updateDatasetsSHA256IndicatorsList "${score}"
done

#update the Datasets indicators alert rules
logInfo "Updating Datasets Rules list file..."
updateDatasetsRulesList

#restart/reload suricata if needed
if [[ $restartSuricata -eq 1 ]]
then
    logInfo "Restarting Suricata...."
    restartSuricataService

    if [[ $? -eq 0 ]]
    then
        triggerSuricataRulesReload=0
    fi
fi

if [[ $triggerSuricataRulesReload -eq 1 ]]
then
    logInfo "Reloading Suricata rules...."
    reloadSuricataRules ${suricata_config_file_path}
fi

logInfo "Finished running Suricata indicators update script."